#!/bin/bash
# setup_ansible_lab.sh

# Define colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
NC='\033[0m' # No Color

# Base directory structure
echo -e "${GREEN}Creating directory structure...${NC}"
mkdir -p ansible-docker-lab/{docker,ansible}/{red,blue,grey}
mkdir -p ansible-docker-lab/docker/keys
mkdir -p ansible-docker-lab/ansible/{inventory,playbooks,roles,group_vars,host_vars}

# Move into project directory
cd ansible-docker-lab

# Generate SSH key for Ansible
echo -e "${GREEN}Generating SSH key for Ansible...${NC}"
ssh-keygen -t ed25519 -C "ansible" -f docker/keys/ansible_key -q -N ""

# Create Dockerfiles
echo -e "${GREEN}Creating Dockerfiles...${NC}"#!/bin/bash

# Base Ubuntu Dockerfile
touch docker/Dockerfile.ubuntu.base
cat > docker/Dockerfile.ubuntu.base << 'EOF'
FROM ubuntu:22.04

# Install essential packages
RUN apt-get update && apt-get install -y \
    openssh-server \
    python3 \
    sudo \
    && rm -rf /var/lib/apt/lists/*

# SSH configuration
RUN mkdir /run/sshd
RUN echo 'PermitRootLogin yes' >> /etc/ssh/sshd_config
RUN echo 'PubkeyAuthentication yes' >> /etc/ssh/sshd_config

# Create ansible user
RUN useradd -m -s /bin/bash ansible \
    && echo "ansible ALL=(ALL) NOPASSWD:ALL" > /etc/sudoers.d/ansible

# Copy SSH key
COPY keys/ansible_key.pub /home/ansible/.ssh/authorized_keys

# Set proper permissions
RUN mkdir -p /home/ansible/.ssh \
    && chown -R ansible:ansible /home/ansible/.ssh \
    && chmod 700 /home/ansible/.ssh \
    && chmod 600 /home/ansible/.ssh/authorized_keys

# Start SSH daemon
CMD ["/usr/sbin/sshd", "-D"]
EOF

# Red Team Web Server
cat > docker/red/Dockerfile.web << 'EOF'
FROM ansible-lab-ubuntu-base

# Install nginx
RUN apt-get update && apt-get install -y nginx \
    && rm -rf /var/lib/apt/lists/*

# Set red team prompt
RUN echo 'PS1="\[\033[1;31m\][\u@\h \W]\\$ \[\033[0m\]"' >> /home/ansible/.bashrc

EXPOSE 80
CMD ["/usr/sbin/sshd", "-D"]
EOF

# Red Team Basic
cat > docker/red/Dockerfile.basic << 'EOF'
FROM ansible-lab-ubuntu-base

# Set red team prompt
RUN echo 'PS1="\[\033[1;31m\][\u@\h \W]\\$ \[\033[0m\]"' >> /home/ansible/.bashrc
EOF

# Blue Team AlmaLinux
cat > docker/blue/Dockerfile.alma << 'EOF'
FROM almalinux:9

# Install essential packages
RUN dnf -y install openssh-server python3 sudo \
    && dnf clean all

# SSH configuration
RUN mkdir /run/sshd
RUN echo 'PermitRootLogin yes' >> /etc/ssh/sshd_config
RUN echo 'PubkeyAuthentication yes' >> /etc/ssh/sshd_config

# Create ansible user
RUN useradd -m -s /bin/bash ansible \
    && echo "ansible ALL=(ALL) NOPASSWD:ALL" > /etc/sudoers.d/ansible

# Copy SSH key
COPY keys/ansible_key.pub /home/ansible/.ssh/authorized_keys

# Set blue team prompt
RUN echo 'PS1="\[\033[1;36m\][\u@\h \W]\\$ \[\033[0m\]"' >> /home/ansible/.bashrc

# Set proper permissions
RUN mkdir -p /home/ansible/.ssh \
    && chown -R ansible:ansible /home/ansible/.ssh \
    && chmod 700 /home/ansible/.ssh \
    && chmod 600 /home/ansible/.ssh/authorized_keys

CMD ["/usr/sbin/sshd", "-D"]
EOF

# Blue Team Basic
cat > docker/blue/Dockerfile.basic << 'EOF'
FROM ansible-lab-ubuntu-base

# Set blue team prompt
RUN echo 'PS1="\[\033[1;36m\][\u@\h \W]\\$ \[\033[0m\]"' >> /home/ansible/.bashrc
EOF

# Grey Team (Both identical)
cat > docker/grey/Dockerfile << 'EOF'
FROM ansible-lab-ubuntu-base

# Set grey/orange team prompt
RUN echo 'PS1="\[\033[1;33m\][\u@\h \W]\\$ \[\033[0m\]"' >> /home/ansible/.bashrc
EOF

# Create docker-compose.yml
cat > docker-compose.yml << 'EOF'
version: '3.8'

services:
  red-web:
    build:
      context: docker
      dockerfile: red/Dockerfile.web
    hostname: red-web
    networks:
      - red_network

  red-basic:
    build:
      context: docker
      dockerfile: red/Dockerfile.basic
    hostname: red-basic
    networks:
      - red_network

  blue-alma:
    build:
      context: docker
      dockerfile: blue/Dockerfile.alma
    hostname: blue-alma
    networks:
      - blue_network

  blue-basic:
    build:
      context: docker
      dockerfile: blue/Dockerfile.basic
    hostname: blue-basic
    networks:
      - blue_network

  grey-1:
    build:
      context: docker
      dockerfile: grey/Dockerfile
    hostname: grey-1
    networks:
      - red_network
      - blue_network
      - grey_network

  grey-2:
    build:
      context: docker
      dockerfile: grey/Dockerfile
    hostname: grey-2
    networks:
      - red_network
      - blue_network
      - grey_network

networks:
  red_network:
    driver: bridge
  blue_network:
    driver: bridge
  grey_network:
    driver: bridge
EOF

# Create basic inventory file
cat > ansible/inventory/hosts << 'EOF'
[red]
red-web
red-basic

[blue]
blue-alma
blue-basic

[grey]
grey-1
grey-2

[all:vars]
ansible_user=ansible
ansible_ssh_private_key_file=../docker/keys/ansible_key
EOF

echo -e "${GREEN}Setup complete! To build and run the lab:${NC}"
echo -e "1. Build the base image: ${RED}docker build -t ansible-lab-ubuntu-base -f docker/Dockerfile.ubuntu.base .${NC}"
echo -e "2. Start the lab: ${RED}docker compose up -d${NC}"
echo -e "3. To stop the lab: ${RED}docker compose down${NC}"
