# Ansible Deep Dive

## What is Ansible and why is it useful?
Ansible is an open-source automation tool that enables infrastructure as code, configuration management, and application deployment. It uses simple YAML syntax and requires no agents on remote systems.

Core concepts:
- Playbooks: YAML files containing automation instructions
- Inventory: List of managed hosts and groups
- Modules: Pre-built units of code for specific tasks
- Roles: Reusable collections of tasks and configurations

Ansible uses SSH for communication and Python for execution, making it agentless and secure by default.

**Key Takeaways:**
- Ansible is agentless and uses existing SSH infrastructure
- YAML syntax makes it human-readable and easy to learn
- Idempotent operations ensure consistent state

## Hands-on with Ansible Commands I

### ansible
The `ansible` command runs ad-hoc commands across hosts.

Exercise 1: Test connectivity
```sh
# Our inventory is already configured at ansible/inventory/hosts
# Test connectivity to all hosts
ansible -i inventory/hosts all -m ping

# Test specific teams
ansible -i inventory/hosts red -m ping
ansible -i inventory/hosts blue -m ping
ansible -i inventory/hosts grey -m ping
```

Exercise 2: Run commands
```sh
# Check disk space on all systems
ansible -i inventory/hosts all -m shell -a "df -h"

# Update package cache (with sudo)
# For Ubuntu hosts
ansible -i inventory/hosts all -m apt -a "update_cache=yes" --become

# For our AlmaLinux host
ansible -i inventory/hosts blue-alma -m dnf -a "update_cache=yes" --become
```

### ansible-playbook
Execute Ansible playbooks.

Exercise 3: Create and run a simple playbook
```yaml
# playbooks/nginx_test.yml
---
- name: Simple Nginx Playbook
  hosts: red-web
  become: yes
  tasks:
    - name: Ensure nginx is installed
      apt:
        name: nginx
        state: present

    - name: Start nginx service
      service:
        name: nginx
        state: started
```

```sh
ansible-playbook -i inventory/hosts playbooks/nginx_test.yml
```

**Key Takeaways:**
- Ad-hoc commands for quick tasks
- Playbooks for repeatable automation
- Modules handle most common operations

## Ansible Inventory
Our lab environment uses a structured inventory:

```ini
# inventory/hosts
[red]
red-web ansible_host=172.22.0.2
red-basic ansible_host=172.22.0.4

[blue]
blue-alma ansible_host=172.23.0.3
blue-basic ansible_host=172.23.0.2

[grey]
grey-1 ansible_host=172.22.0.5
grey-2 ansible_host=172.22.0.6

[all:vars]
ansible_user=ansible
ansible_ssh_private_key_file=/path/to/docker/keys/ansible_key
```

Exercise 4: Test inventory groups
```sh
# List all hosts
ansible -i inventory/hosts all --list-hosts

# List red team hosts
ansible -i inventory/hosts red --list-hosts

# Get facts about a specific host
ansible -i inventory/hosts red-web -m setup
```

Exercise 5: Group variables
```yaml
# group_vars/red.yml
---
team_name: Red Team
http_port: 80
enable_ssl: false

# group_vars/all.yml
---
timezone: "UTC"
```

**Key Takeaways:**
- Our lab has three distinct networks
- Grey team hosts can access both red and blue networks
- Static IPs make host addressing consistent

## Writing Effective Playbooks

Exercise 6: Multi-play playbook for our lab
```yaml
# playbooks/lab_setup.yml
---
- name: Configure red web server
  hosts: red-web
  become: yes
  tasks:
    - name: Install nginx
      apt:
        name: nginx
        state: present

    - name: Create test webpage
      copy:
        content: "Welcome to Red Team Web Server\n"
        dest: /var/www/html/index.html
        mode: '0644'

- name: Configure AlmaLinux host
  hosts: blue-alma
  become: yes
  tasks:
    - name: Install required packages
      dnf:
        name:
          - httpd
          - vim
        state: present
```

Exercise 7: Using handlers with our web server
```yaml
# handlers.yml
---
- hosts: red-web
  become: yes
  tasks:
    - name: Update nginx config
      copy:
        content: |
          server {
            listen 80;
            root /var/www/html;
            index index.html;
          }
        dest: /etc/nginx/sites-available/default
      notify: Restart nginx

  handlers:
    - name: Restart nginx
      service:
        name: nginx
        state: restarted
```


## Final Thoughts
Ansible provides a powerful yet simple approach to automation. Understanding these fundamentals enables efficient infrastructure management and application deployment at scale.

## Practice Exercises

### Basic Automation

Create a playbook that:
- Updates system packages
- Installs a web server
- Deploys a simple HTML file
- Ensures the service is running

### Role Development

Create a role that:
- Installs and configures a database
- Creates a database and user
- Implements proper security measures
- Uses templates for configuration

### Multi-Environment Setup

Demonstrate environment management:
- Create development and production inventories
- Use group variables for environment-specific settings
- Implement different security measures per environment
- Test playbook execution in both environments

### Vault Integration

Create a secure deployment:
- Store sensitive data in Ansible Vault
- Create a playbook that uses these secrets
- Implement proper key management
- Document the deployment process

Remember to test each exercise in a safe environment!