# Docker Deep Dive

## What is Docker and why is it useful?
Docker is a platform for developing, shipping, and running applications in containers. It provides a consistent environment across different stages of development and deployment.

Core concepts:
- Images: Read-only templates containing application code, runtime, libraries, and dependencies
- Containers: Running instances of images that can be started, stopped, and deleted
- Docker Hub: Cloud-based registry for sharing and managing Docker images

Docker uses a client-server architecture where the Docker daemon (server) processes commands sent through the Docker CLI (client).

**Key Takeaways:**

- Docker solves the "it works on my machine" problem
- Containers are lightweight and share the host OS kernel
- Docker Hub provides access to thousands of pre-built images

## Hands-on with Docker Commands I

### docker run
The `docker run` command creates and starts a container from an image.

Common options:
- `-d`: Run container in detached mode (background)
- `-it`: Interactive mode with terminal access
- `-p host:container`: Port mapping
- `--name`: Assign a name to the container

Exercise 1: Run hello-world container
```sh
docker run hello-world
```

Exercise 2: Run Nginx container
```sh
docker run -d -p 8080:80 --name my-nginx nginx
# Access http://localhost:8080 in your browser
```

### docker ps
View running containers and their details.

Exercise 3: List containers
```sh
docker ps        # List running containers
docker ps -a     # List all containers including stopped ones
```

### docker stop & docker start
Manage container lifecycle.

Exercise 4: Container lifecycle management
```sh
docker stop my-nginx
docker start my-nginx
```

### docker rm
Remove containers.

Exercise 5: Remove containers
```sh
docker stop my-nginx     # Stop if running
docker rm my-nginx
docker container prune
docker rm $(docker ps -aq)  # Remove all stopped containers
```

### docker logs
View container logs.

Exercise 6: Log inspection
```sh
docker run -d --name log-test alpine sh -c 'while true; do echo "Log entry at $(date)"; sleep 3; done'
docker logs log-test
docker logs -f log-test  # Follow log output
```

**Key Takeaways:**
- Container lifecycle: create, start, stop, remove
- Containers can run in interactive or detached mode
- Port mapping enables access to container services
- Logs help debug container issues

## Hands-on with Docker Commands II

### docker images
List local images.

Exercise 7: Image management
```sh
docker images
docker images -a  # Show all images including intermediates
```

### docker pull
Download images from Docker Hub.

Exercise 8: Pull Ubuntu image
```sh
docker pull almalinux/almalinux
```

### docker exec
Execute commands in running containers.

Exercise 9: Interactive container management
```sh
docker run -d --name alma-test almalinux/almalinux tail -f /dev/null
docker exec -it alma-test bash
yum update  # Inside container
exit
```

**Key Takeaways:**
- Images are the building blocks for containers
- Version tags help manage different image releases
- exec enables interaction with running containers

## Introduction to Dockerfile
A Dockerfile is a text file containing instructions to build a Docker image.

Common instructions:
- FROM: Base image
- RUN: Execute commands during build
- COPY: Copy files from host to image
- WORKDIR: Set working directory
- EXPOSE: Document container ports
- CMD: Default command to run
- ENTRYPOINT: Container entry point

Best practices:
- Use specific base image versions
- Combine RUN commands to reduce layers
- Remove unnecessary files
- Use .dockerignore

## Building Images with Dockerfile

Exercise 10: Create and build Python application
```sh
# Create app.py
echo 'print("Hello from Python!")' > app.py

# Create Dockerfile
cat << EOF > Dockerfile
FROM python:3.9-slim
WORKDIR /app
COPY app.py .
CMD ["python", "app.py"]
EOF

# Build and run
docker build -t my-python-app .
docker run my-python-app
```

Exercise 11: Tag and organize images
```sh
docker tag my-python-app username/my-python-app:v1.0
```

**Key Takeaways:**
- Dockerfiles automate image creation
- Tags help version and organize images
- Multi-stage builds optimize image size

## Docker Networking Basics

Exercise 12: Container networking
```sh
# Create network
docker network create my-network

# Run containers in network
docker run -d --name container1 --network my-network alpine tail -f /dev/null
docker run -d --name container2 --network my-network alpine tail -f /dev/null

# Test communication
docker exec container1 ping -c 3 container2
```

**Key Takeaways:**
- Containers can communicate via networks
- Network isolation improves security
- Custom networks enable service discovery

## Introduction to Docker Registry

Exercise 13: Registry interaction
```sh
docker login
docker push username/my-python-app:v1.0
docker pull username/my-python-app:v1.0
```

**Key Takeaways:**
- Registries store and distribute images
- Docker Hub is the default public registry
- Private registries enable secure image storage


## Final Thoughts
Docker provides a powerful platform for containerization, enabling consistent development and deployment workflows. Understanding these core concepts and commands forms the foundation for more advanced Docker usage in production environments.

## Practice Exercises

### Container Lifecycle

Create a workflow that demonstrates your understanding of container lifecycle:

- Pull and run an Ubuntu container in detached mode
- Execute a command to create a file inside the container
- Stop the container
- Start it again
- Verify your file still exists
- Clean up all resources

### Web Server Challenge

Using what you've learned about port mapping:
- Run an nginx container
- Map it to port 8080 on your host
- Verify you can access the default page
- View the container logs
- Stop and remove the container

### Volume Sharing Challenge

Demonstrate your understanding of container data persistence:
- Create a directory on your host machine with a text file
- Run a container that mounts this directory as a volume
- Modify the text file from inside the container
- Verify the changes are visible on your host machine
- Create a new file inside the container
- Verify it appears on your host machine

### Basic Dockerfile Creation

Create a Dockerfile that:
- Uses Python as the base image
- Creates a simple script that prints "Hello, Docker!"
- Build and run your image
- Push it to Docker Hub (optional)

Remember to document the commands you use for each exercise!