# Terraform Deep Dive

## What is Terraform and why is it useful?
Terraform is an Infrastructure as Code (IaC) tool that allows you to define and provision infrastructure using declarative configuration files. It supports multiple cloud providers and services through its provider system.

Core concepts:
- Providers: Plugins that enable interaction with cloud platforms, services, and APIs
- Resources: Infrastructure objects you want to manage
- State: Terraform's record of managed infrastructure
- Variables: Reusable values that can be input into configurations
- Outputs: Values that can be queried or exported

**Key Takeaways:**
- Terraform enables version-controlled infrastructure
- Configurations are provider-agnostic
- State management ensures consistency
- Modular design promotes reusability

## Hands-on with Terraform Commands I

### terraform init

Initializes a Terraform working directory, downloading providers and modules.

Exercise 1: Initialize a project
```sh
cd /root/woac-infra-demo/ranges/terraform/terraform-test
terraform init
```

### terraform plan

Shows execution plan for infrastructure changes.

Exercise 2: Create and review a plan

`terraform plan`

### terraform apply

Applies the changes required to reach desired state.

Exercise 3: Apply configuration
```sh
terraform apply
terraform apply -auto-approve  # Skip approval prompt
```

### terraform destroy

Removes all resources managed by the configuration.

Exercise 4: Clean up resources
```sh
terraform destroy
```

**Key Takeaways:**
- Workflow: init → plan → apply → destroy
- Plan provides safety check before changes
- State tracks resource relationships
- Destroy removes managed resources



## Terraform Configuration Basics

### Provider Configuration

```yaml
terraform {
  required_providers {
    openstack = {
      source = "terraform-provider-openstack/openstack"
      version = "3.0.0"
    }
  }
}
```

### Resource Blocks

```yaml
resource "openstack_compute_instance_v2" "example" {
  name = "example-instance"
  image_name = "Ubuntu 20.04"
  flavor_name = "m1.tiny"
  network {
    name = "public"
  }

  tags = {
    Name = "example-instance"
  }
}
```

### Variables

```yaml
# variables.tf
variable "instance_type" {
  description = "EC2 instance type"
  type        = string
  default     = "t2.micro"
}

# main.tf usage
resource "aws_instance" "example" {
  instance_type = var.instance_type
}
```

### Outputs

```yaml
output "instance_ip" {
  value = aws_instance.example.public_ip
}
```

## Working with State

State management

```sh
terraform show           # Display current state
terraform state list    # List resources
terraform state pull    # Download remote state
terraform state push    # Upload local state
```

**Key Takeaways:**
- State should be stored remotely in production
- Sensitive data in state should be encrypted
- State files should be backed up
- Team members need shared state access

## Terraform Workspaces

 Managing environments

```sh
terraform workspace new dev
terraform workspace new prod
terraform workspace select dev
```

## Data Sources


```yaml
data "openstack_compute_image_v2" "ubuntu" {
  most_recent = true
  name = "Ubuntu 20.04"
  filter {
    name   = "name"
    values = ["ubuntu/images/hvm-ssd/ubuntu-focal-20.04-amd64-server-*"]
  }
}
```

## Modules

Create reusable module
```yaml
# modules/webserver/main.tf
variable "instance_type" {}

resource "openstack_compute_instance_v2" "web" {
  image_name = "Ubuntu 20.04"
  flavor_name = "m1.tiny"
  network {
    name = "public"
  }
}

# Root main.tf
module "web_server" {
  source        = "./modules/webserver"
  instance_type = "t2.micro"
}
```

**Key Takeaways:**
- Modules promote code reuse
- Workspaces separate environments
- Data sources query existing resources
- Variables enable configuration

## Best Practices

1. Code Organization
- Use consistent formatting (terraform fmt)
- Separate variables and outputs
- Group related resources
- Use modules for reusability

2. State Management
- Use remote state
- Enable state locking
- Backup state files
- Use workspaces for environments

3. Security
- Use variables for sensitive data
- Encrypt state files
- Use least privilege access
- Implement proper IAM roles

## Practice Exercises

### Challenge 1
Create a single file terraform representation of our stu-ops.yaml heat template

For resource documentation see [here](https://registry.terraform.io/providers/terraform-provider-openstack/openstack/latest/docs)

### Challenge 2

For additional practice (which utilizes docker) follow the tutorial [here](https://developer.hashicorp.com/terraform/tutorials/docker-get-started/infrastructure-as-code)